@@ -16,6 +16,8 @@ side querying and analysis with the support of an array
 of scientific computing python libraries, such as ipython,
 pandas, numpy, matplotlib, and more.
 
+The main client interface is `metrique.pyclient`
+
 A simple example of how one might interact with metrique is
 demonstrated below. In short, we import one of the many
 pre-defined metrique cubes -- `osinfo_rpm` -- in this case.
@@ -61,17 +63,12 @@ And finishing with some querying and simple charting of the data.
     >>> sub.plot(kind='bar')
     ... <matplotlib.axes.AxesSubplot at 0x6f77ad0>
 
-:copyright: 2013 "Chris Ward" <cward@redhat.com>
-:license: GPLv3, see LICENSE for more details
-:sources: https://github.com/drpoovilleorg/metrique
-
 .. note::
     example date ranges: 'd', '~d', 'd~', 'd~d'
 .. note::
     valid date format: '%Y-%m-%d %H:%M:%S,%f', '%Y-%m-%d %H:%M:%S', '%Y-%m-%d'
 '''
 
-from collections import MutableSequence
 from copy import copy
 import cPickle
 import gc
@@ -91,31 +88,31 @@ from metrique.config import Config
 from metrique.utils import json_encode, get_cube
 from metriqueu.utils import utcnow
 
-# setup default root logger, but remove default StreamHandler (stderr)
-# Handlers will be added upon __init__()
-logging.basicConfig()
-root_logger = logging.getLogger()
-[root_logger.removeHandler(hdlr) for hdlr in root_logger.handlers]
-BASIC_FORMAT = "%(name)s:%(asctime)s:%(message)s"
+logger = logging.getLogger(__name__)
+
 FILETYPES = {'csv': pd.read_csv, 'json': pd.read_json}
 fields_re = re.compile('[\W]+')
 space_re = re.compile('\s+')
 unda_re = re.compile('_')
 
 
-class BaseCube(MutableSequence):
+class BaseClient(object):
     '''
-    Default 'cube' container model. Essentially, this object is made from
-    a list of dicts. The underlying object inherits from python's
-    MutableSequence collection object (aka, list).
+    Low level client API which provides baseline functionality, including
+    methods for loading data from csv and json, loading metrique client
+    cubes, config file loading and logging setup.
+
+    Essentially, cubes are data made from a list of dicts.
 
     All objects are expected to contain a `_oid` key value property. This
-    property should be unique per individual "object" defined. For example,
-    if we are storing logs, we might consider each log line a separate
-    "object" since those log lines should never change in the future and give
-    each a unique `_oid`. Or if we are storing data about 'meta objects' of
-    some sort, say 'github repo issues' for example, we might have objects
-    with _oids of '%(username)s_%(reponame)s_%(issuenumber)s'.
+    property should be unique per individual "object" defined.
+
+    For example, if we are storing logs, we might consider each log line a
+    separate "object" since those log lines should never change in the future
+    and give each a unique `_oid`. Or if we are storing data about
+    'meta objects' of some sort, say 'github repo issues' for example, we
+    might have objects with _oids of
+    `%(username)s_%(reponame)s_%(issuenumber)s`.
 
     Optionally, objects can contain the following additional meta-properties:
         * _start - datetime when the object state was set
@@ -124,7 +121,7 @@ class BaseCube(MutableSequence):
     Field names (object dict keys) must consist of alphanumeric and underscore
     characters only.
 
-    Field names are normalized automatically:
+    Field names are partially normalized automatically:
         * non-alphanumeric characters are removed
         * spaces converted to underscores
         * letters are lowercased
@@ -132,99 +129,162 @@ class BaseCube(MutableSequence):
     Property values are normalized to some extent automatically as well:
         * empty strings -> None
 
+    Additionally, some common operation methods are provided for
+    operations such as loading a HTTP uri and determining currently
+    configured username.
+
+    :cvar name: name of the cube
+    :cvar defaults: cube default property container (cube specific meta-data)
+    :cvar fields: cube fields definitions
+    :cvar saveas: filename to use when saving cube data to disk locally
+    :cvar config: local cube config object
+
+    If cube is specified as a kwarg upon initialization, the specific cube
+    class will be located and returned, assuming its available in sys.path.
+
+    If the cube fails to import, RuntimeError will be raised.
+
+    Example usage::
+
+        >>> import pyclient
+        >>> c = pyclient(cube='git_commit')
+            <type HTTPClient(...)>
+
+        >>> z = pyclient()
+        >>> z.get_cube(cube='git_commit')
+            <type HTTPClient(...)>
+
     '''
-    _objects = []
+    name = None
+    defaults = None
+    fields = None
+    saveas = ''
+    _cache = None
+    config = None
 
-    def __init__(self, name, objects=None):
-        if name:
-            self.name = name
+    def __new__(cls, *args, **kwargs):
+        if 'cube' in kwargs and kwargs['cube']:
+            cls = get_cube(cube=kwargs['cube'], init=False)
+        else:
+            cls = cls
+        return object.__new__(cls)
+
+    def __init__(self, config_file=None, name=None, **kwargs):
+        # don't assign to {} in class def, define here to avoid
+        # multiple pyclient objects linking to a shared dict
+        if self.defaults is None:
+            self.defaults = {}
+        if self.fields is None:
+            self.fields = {}
+        if self._cache is None:
+            self._cache = {}
+        if self.config is None:
+            self.config = {}
+
+        self._config_file = config_file or Config.default_config
+
+        # all defaults are loaded, unless specified in
+        # metrique_config.json
+        self.load_config(**kwargs)
+
+        # cube class defined name
+        self._cube = type(self).name
+
+        utc_str = utcnow(as_datetime=True).strftime('%a%b%d%H%m%S')
+        # set name if passed in, but don't overwrite default if not
+        self.name = name or self.name or utc_str
+
+        self.config.logdir = os.path.expanduser(self.config.logdir)
+        if not os.path.exists(self.config.logdir):
+            os.makedirs(self.config.logdir)
+        self.config.logfile = os.path.join(self.config.logdir,
+                                           self.config.logfile)
+
+        # keep logging local to the cube so multiple
+        # cubes can independently log without interferring
+        # with each others logging.
+        self.debug_setup()
+
+####################### data loading api ###################
+    def load_files(self, path, filetype=None, **kwargs):
+        '''Load multiple files from various file types automatically.
+
+        Supports glob paths, eg::
+
+            path = 'data/*.csv'
+
+        Filetypes are autodetected by common extension strings.
+
+        Currently supports loadings from:
+            * csv (pd.read_csv)
+            * json (pd.read_json)
+
+        :param path: path to config json file
+        :param filetype: override filetype autodetection
+        :param kwargs: additional filetype loader method kwargs
+        '''
+        # kwargs are for passing ftype load options (csv.delimiter, etc)
+        # expect the use of globs; eg, file* might result in fileN (file1,
+        # file2, file3), etc
+        datasets = glob.glob(os.path.expanduser(path))
+        objects = None
+        for ds in datasets:
+            filetype = path.split('.')[-1]
+            # buid up a single dataframe by concatting
+            # all globbed files together
+            objects = pd.concat(
+                [self._load_file(ds, filetype, **kwargs)
+                    for ds in datasets]).T.as_dict().values()
+        return objects
+
+    def _load_file(self, path, filetype, **kwargs):
+        if filetype in ['csv', 'txt']:
+            return self._load_csv(path, **kwargs)
+        elif filetype in ['json']:
+            return self._load_json(path, **kwargs)
+        else:
+            raise TypeError("Invalid filetype: %s" % filetype)
+
+    def _load_csv(self, path, **kwargs):
+        # load the file according to filetype
+        return pd.read_csv(path, **kwargs)
+
+    def _load_json(self, path, **kwargs):
+        return pd.read_json(path, **kwargs)
+
+####################### objects manipulation ###################
+    def df(self, objects=None):
+        '''Return a pandas dataframe from objects'''
         if objects:
-            self.objects = objects
-
-    def __delitem__(self, name):
-        del self.objects[name]
-
-    def __getitem__(self, name):
-        return self.objects[name]
-
-    def __setitem__(self, name, obj):
-        self.objects[name] = obj
-
-    def insert(self, name, obj):
-        self.objects.insert(name, obj)
-
-    def __len__(self):
-        return len(self.objects)
-
-    def __iter__(self):
-        return iter(self.objects)
-
-    def __next__(self):
-        yield next(self)
-
-    def next(self):
-        return self.__next__()
-
-    def __getslice__(self, i, j):
-        return self.objects[i:j]
-
-    def __contains__(self, item):
-        return item in self.objects
-
-    def __str__(self):
-        return str(self.objects)
-
-    def __repr__(self):
-        return repr(self.objects)
-
-####################################################################
-    @property
-    def df(self):
-        if self.objects:
-            return pd.DataFrame(self.objects)
+            return pd.DataFrame(objects)
         else:
             return pd.DataFrame()
 
     def flush(self):
-        del self.objects
+        '''run garbage collection'''
+        k = gc.get_count()
+        result = gc.collect()  # be sure we garbage collect any old object refs
+        logger.debug('Garbage Flush: %s flushed, %s remain' % (k, result))
 
-    @property
-    def objects(self):
-        return self._objects
-
-    @objects.setter
-    def objects(self, objects):
+    def normalize(self, objects):
+        '''Convert, validate, normalize and locally cache objects'''
         # convert from other forms to basic list of dicts
         if objects is None:
             objects = []
         elif isinstance(objects, pd.DataFrame):
             objects = objects.T.to_dict().values()
-        elif isinstance(objects, BaseCube):
-            objects = objects.objects
-        # model check
-        if not isinstance(objects, (BaseCube, list, tuple)):
-            _t = type(objects)
-            raise TypeError("container value must be a list; got %s" % _t)
+        elif isinstance(objects, tuple):
+            objects = list(objects)
+        else:
+            assert isinstance(objects, list)
         if objects:
             if not all([type(o) is dict for o in objects]):
                 raise TypeError("object values must be dict")
             if not all([o.get('_oid') is not None for o in objects]):
                 raise ValueError("_oid must be defined for all objs")
-            self._objects = self._normalize(objects)
-        else:
-            self._objects = objects
+            objects = self._normalize(objects)
+        return objects
 
-    @objects.deleter
-    def objects(self):
-        del self._objects
-        self._objects = []
-        gc.collect()  # be sure we garbage collect any old object refs
-
-    @property
-    def oids(self):
-        return [o['_oid'] for o in self._objects]
-
-###################### normalization keys/values #################
     def _normalize(self, objects):
         '''
         give all these objects the same _start value (if they
@@ -267,148 +327,50 @@ class BaseCube(MutableSequence):
         obj['_start'] = _start or utcnow()
         return obj
 
-
-class BaseClient(BaseCube):
-    '''
-    Low level client API
-
-    Functionality includes methods for loading data from csv and json,
-    loading metrique client cubes, config file loading and logging
-    setup.
-
-    Additionally, some common operation methods are provided for
-    operations such as loading a HTTP uri and determining currently
-    configured username.
-    '''
-    name = None
-    # defaults is frequently overrided in subclasses as a property
-    defaults = None
-    # fields is frequently overrided in subclasses as a property too
-    fields = None
-    # filename of the data when saved to disk
-    saveas = ''
-    # a place to put stuff, temporarily...
-    _cache = None
-    # config properties
-    config = {}
-
-    def __new__(cls, *args, **kwargs):
-        '''
-        Return the specific cube class, if specified. Its
-        expected the cube will be available in sys.path.
-
-        If the cube fails to import, just move on.
-
-            >>> import pyclient
-            >>> c = pyclient(cube='git_commit')
-                <type HTTPClient(...)>
-        '''
-        if 'cube' in kwargs and kwargs['cube']:
-            cls = get_cube(cube=kwargs['cube'], init=False)
-        else:
-            cls = cls
-        return object.__new__(cls)
-
-    def __init__(self, config_file=None, name=None, objects=None, **kwargs):
-        # don't assign to {} in class def, define here to avoid
-        # multiple pyclient objects linking to a shared dict
-        if self.defaults is None:
-            self.defaults = {}
-        if self.fields is None:
-            self.fields = {}
-        if self._cache is None:
-            self._cache = {}
-
-        self._config_file = config_file or Config.default_config
-
-        # all defaults are loaded, unless specified in
-        # metrique_config.json
-        self.load_config(**kwargs)
-
-        utc_str = utcnow(as_datetime=True).strftime('%a%b%d%H%m%S')
-        # set name if passed in, but don't overwrite default if not
-        self.name = name or self.name or utc_str
-
-        self.objects = BaseCube(name=self.name, objects=objects)
-
-        self.config.logdir = os.path.expanduser(self.config.logdir)
-        if not os.path.exists(self.config.logdir):
-            os.makedirs(self.config.logdir)
-        self.config.logfile = os.path.join(self.config.logdir,
-                                           self.config.logfile)
-
-        # keep logging local to the cube so multiple
-        # cubes can independently log without interferring
-        # with each others logging.
-        self.debug_set()
-
-####################### data loading api ###################
-    def load_files(self, path, filetype=None, **kwargs):
-        # kwargs are for passing ftype load options (csv.delimiter, etc)
-        # expect the use of globs; eg, file* might result in fileN (file1,
-        # file2, file3), etc
-        datasets = glob.glob(os.path.expanduser(path))
-        for ds in datasets:
-            filetype = path.split('.')[-1]
-            # buid up a single dataframe by concatting
-            # all globbed files together
-            self.objects = pd.concat(
-                [self.load_file(ds, filetype, **kwargs)
-                    for ds in datasets]).T.as_dict().values()
-        return self.objects
-
-    def load_file(self, path, filetype, **kwargs):
-        if filetype in ['csv', 'txt']:
-            return self.load_csv(path, **kwargs)
-        elif filetype in ['json']:
-            return self.load_json(path, **kwargs)
-        else:
-            raise TypeError("Invalid filetype: %s" % filetype)
-
-    def load_csv(self, path, **kwargs):
-        # load the file according to filetype
-        return pd.read_csv(path, **kwargs)
-
-    def load_json(self, path, **kwargs):
-        return pd.read_json(path, **kwargs)
+    def oids(self, objects):
+        '''Return back a list of _oids for all locally cached objects'''
+        return [o['_oid'] for o in objects]
 
 #################### misc ##################################
-    def debug_set(self, level=None, logstdout=None, logfile=None):
+    def debug_setup(self):
         '''
-        if we get a level of 2, we want to apply the
-        debug level to all loggers
+        Local object instance logger setup.
+
+        Verbosity levels are determined as such::
+
+            if level in [-1, False]:
+                logger.setLevel(logging.WARN)
+            elif level in [0, None]:
+                logger.setLevel(logging.INFO)
+            elif level in [True, 1, 2]:
+                logger.setLevel(logging.DEBUG)
+
+        If (level == 2) `logging.DEBUG` will be set even for
+        the "root logger".
+
+        Configuration options available for customized logger behaivor:
+            * debug (bool)
+            * logstdout (bool)
+            * log2file (bool)
+            * logfile (path)
         '''
-        if level is None:
-            level = self.config.debug
-        if logstdout is None:
-            logstdout = self.config.logstdout
-        if logfile is None:
-            logfile = self.config.logfile
+        level = self.config.debug
+        logstdout = self.config.logstdout
+        logfile = self.config.logfile
+        log_format = "%(name)s.%(process)s:%(asctime)s:%(message)s"
+        log_format = logging.Formatter(log_format, "%Y%m%dT%H%M%S")
 
-        basic_format = logging.Formatter(BASIC_FORMAT)
-
-        if level == 2:
-            self._logger_name = None
-            logger = logging.getLogger()
-            logger = self._debug_set_level(logger, level)
-
-        self._logger_name = '%s.%s' % ('metrique', self.name)
-        logger = logging.getLogger(self._logger_name)
-        logger.propagate = 0
-
-        logger.handlers = []  # reset handlers
+        logger = logging.getLogger()
+        logger.handlers = []
         if logstdout:
             hdlr = logging.StreamHandler()
-            hdlr.setFormatter(basic_format)
+            hdlr.setFormatter(log_format)
             logger.addHandler(hdlr)
-
         if self.config.log2file and logfile:
             hdlr = logging.FileHandler(logfile)
-            hdlr.setFormatter(basic_format)
+            hdlr.setFormatter(log_format)
             logger.addHandler(hdlr)
-
-        logger = self._debug_set_level(logger, level)
-        self.logger = logger
+        self._debug_set_level(logger, level)
 
     def _debug_set_level(self, logger, level):
         if level in [-1, False]:
@@ -420,15 +382,47 @@ class BaseClient(BaseCube):
         return logger
 
     def get_cube(self, cube, init=True, name=None, **kwargs):
-        ' wrapper for utils.get_cube(); try to load a cube, pyclient '
+        '''wrapper for :func:`metriqueu.utils.get_cube`
+
+        Locates and loads a metrique cube
+
+        :param cube: name of cube to load
+        :param init: (bool) initialize cube before returning?
+        :param name: override the name of the cube
+        :param kwargs: additional :func:`metriqueu.utils.get_cube`
+        '''
         config = copy(self.config)
         # don't apply the name to the current obj, but to the object
         # we get back from get_cube
         return get_cube(cube=cube, init=init, config=config,
                         name=name, **kwargs)
 
+    def get_property(self, property, field=None, default=None):
+        '''Lookup cube defined property (meta-data):
+
+            1. First try to use the field's property, if defined.
+            2. Then try to use the default property, if defined.
+            3. Then use the default for when neither is found.
+            4. Or return None, if no default is defined.
+
+        :param property: property key name
+        :param field: (optional) specific field to query first
+        :param default: default value to return if [field.]property not found
+        '''
+        try:
+            return self.fields[field][property]
+        except KeyError:
+            try:
+                return self.defaults[property]
+            except (TypeError, KeyError):
+                return default
+
     def load_config(self, config=None, **kwargs):
-        ' try to load a config file and handle when its not available '
+        '''Try to load a config file and handle when its not available
+
+        :param config: config file or :class:`metriqueu.jsonconf.JSONConf`
+        :param kwargs: additional config key:value pairs to store
+        '''
         if type(config) is type(Config):
             self._config_file = config.config_file
         else:
@@ -438,10 +432,11 @@ class BaseClient(BaseCube):
 
 #################### Helper API ############################
     def urlretrieve(self, uri, saveas):
+        '''urllib.urlretrieve wrapper'''
         return urllib.urlretrieve(uri, saveas)
 
     def whoami(self, auth=False):
-        ' quick way of checking the username the instance is working as '
+        '''Local api call to check the username of running user'''
         return self.config['username']
 
 
@@ -554,15 +549,20 @@ class HTTPClient(BaseClient):
 
         if self.logged_in and cube_autoregister:
             if not self.cube_id in self.cube_list_all():
-                self.logger.info("Autoregistering %s" % self.name)
+                logger.info("Autoregistering %s" % self.name)
                 self.cube_register()
 
+    def keys(self, samplesize=1):
+        return self.cube_sample_fields(sample_size=samplesize)
+
+######################### pyclient base API #######################
     @property
     def cube_id(self):
+        '''Return the common cube id string; ie, `%(owner)s__%(name)s`'''
         return '__'.join((self.owner, self.name))
 
     def _build_runner(self, kind, kwargs):
-        ''' generic caller for HTTP
+        '''Generic caller for HTTP
             A) POST; use data, not params
             B) otherwise; use params
         '''
@@ -585,27 +585,28 @@ class HTTPClient(BaseClient):
         return urls
 
     def cookiejar_clear(self):
+        '''Delete existing user cookiejar, if it exists'''
         path = '%s.%s' % (self.config.cookiejar, self.config.username)
         if os.path.exists(path):
             os.remove(path)
 
     def cookiejar_load(self):
+        '''Loading existing user cookiejar, if it exists'''
         path = '%s.%s' % (self.config.cookiejar, self.config.username)
-        cfd = requests.utils.cookiejar_from_dict
         if os.path.exists(path):
             try:
                 with open(path) as cj:
-                    cookiejar = cfd(cPickle.load(cj))
+                    cookiejar = cPickle.load(cj)
             except Exception:
                 pass
             else:
-                self.session.cookies = cookiejar
+                self.session.cookies.update(cookiejar)
 
     def cookiejar_save(self):
+        '''Save current session cookies to cookiejar, if possible'''
         path = '%s.%s' % (self.config.cookiejar, self.config.username)
-        dfc = requests.utils.dict_from_cookiejar
         with open(path, 'w') as f:
-            cPickle.dump(dfc(self.session.cookies), f)
+            cPickle.dump(self.session.cookies, f)
 
     def _delete(self, *args, **kwargs):
         ' requests DELETE; using current session '
@@ -615,20 +616,39 @@ class HTTPClient(BaseClient):
         ' requests GET; using current session '
         return self._run(self.session.get, *args, **kwargs)
 
-    def get_objects(**kwargs):
-        raise NotImplementedError
+    def get_objects(self, objects=None, save=False, autosnap=True):
+        '''Main API method for sub-classed cubes to override for the
+        generation of the objects which are to (potentially) be added
+        to the cube (assuming no duplicates)
 
-    def extract(self, *args, **kwargs):
-        self.get_objects(*args, **kwargs)  # default: stores into self.objects
-        self.cube_save()  # default: saves from self.objects
-        return
+        Basic functionality is to normalize and potentially save objects
+        '''
+        objects = objects or []
+        objects = self.normalize(objects)
+        if save:
+            self.cube_save(objects, autosnap=autosnap)
+        return objects
+
+    def extract(self, autosnap=True, save=True, *args, **kwargs):
+        '''Wrapper of get_objects -> cube_save. Generate all objects
+        then save/persist them to external metriqued host
+
+        :param args: args to pass to `get_objects`
+        :param kwargs: args to pass to `get_objects`
+
+        This method is obsolete; use get_objects() instead.
+        '''
+        return self.get_objects(save=save, autosnap=autosnap, *args, **kwargs)
 
     def get_cmd(self, owner, cube, api_name=None):
-        '''
-        another helper for building api urls, specifically
+        '''Helper method for building api urls, specifically
         for the case where the api call always requires
         owner and cube; api_name is usually provided,
         if there is a 'command name'; but it's optional.
+
+        :param owner: cube owner name
+        :param cube: cube name
+        :param api_name: relative api path
         '''
         owner = owner or self.owner
         if not owner:
@@ -642,9 +662,10 @@ class HTTPClient(BaseClient):
             return os.path.join(owner, cube)
 
     def get_last_field(self, field):
-        '''
-        shortcut for querying to get the last field value for
+        '''Shortcut for querying to get the last field value for
         a given owner, cube.
+
+        :param field: field name to query
         '''
         # FIXME: these "get_*" methods are assuming owner/cube
         # are "None" defaults; ie, that the current instance
@@ -654,27 +675,9 @@ class HTTPClient(BaseClient):
                          sort=[(field, -1)], one=True, raw=True)
         if last:
             last = last.get(field)
-        self.logger.debug(
-            "last %s.%s: %s" % (self.name, field, last))
+        logger.debug("last %s.%s: %s" % (self.name, field, last))
         return last
 
-    def get_property(self, property, field=None, default=None):
-        '''
-        First try to use the field's property, if defined
-        Then try to use the default property, if defined
-        Then use the default for when neither is found
-        Or None, if no default is defined
-
-        OBSOLETE: use metriqueu.utils.set_default
-        '''
-        try:
-            return self.fields[field][property]
-        except KeyError:
-            try:
-                return self.defaults[property]
-            except (TypeError, KeyError):
-                return default
-
     def _get_response(self, runner, _url, username, password,
                       allow_redirects=True, stream=False):
         ' wrapper for running a metrique api request; get/post/etc '
@@ -696,7 +699,7 @@ class HTTPClient(BaseClient):
             content = _response.content
             code = _response.status_code
             content = '[%s] %s\n%s\n%s' % (code, _url, str(e), content)
-            self.logger.error(content)
+            logger.error(content)
             raise
         return _response
 
@@ -712,10 +715,21 @@ class HTTPClient(BaseClient):
         self.cookiejar_load()
 
     def ping(self, auth=False):
-        '''
-        global...base api call; all metrique servers will be expected
+        '''Base api call; all metriqued servers will be expected
         to have this method available. auth=True is a quick way to
         test clients credentials.
+
+        :param auth: (bool) login/authenticate before pinging?
+
+        Example Usage::
+
+            >>> from metrique import pyclient
+            >>> m = pyclient()
+            >>> m.ping()
+                {'action': 'ping', 'metriqued': '127.0.0.1'}
+            >>> m.ping(auth=True)
+                {'action': 'ping', 'metriqued': '127.0.0.1',
+                 'current_user': 'cward'}
         '''
         return self._get('ping', auth=auth)
 
@@ -738,17 +752,18 @@ class HTTPClient(BaseClient):
 
         urls = self._build_urls(cmd, api_url)
         for url in urls:
+            logger.debug("Connecting to %s" % url)
             try:
                 _response = self._get_response(runner, url,
                                                username, password,
                                                allow_redirects,
                                                stream)
             except requests.exceptions.ConnectionError:
-                self.logger.error("Failed to connect to %s" % url)
+                logger.error("Failed to connect to %s" % url)
                 # try the next url available
                 continue
             else:
-                self.logger.debug("Got response from %s" % url)
+                logger.debug("Got response from %s" % url)
 
             if full_response:
                 return _response
@@ -765,23 +780,32 @@ class HTTPClient(BaseClient):
                 except Exception as e:
                     m = getattr(e, 'message')
                     content = '%s\n%s\n%s' % (url, m, _response.content)
-                    self.logger.error(content)
+                    logger.error(content)
                     raise
         else:
             msg = 'Failed to connect to metriqued hosts [%s]' % urls
             raise requests.exceptions.ConnectionError(msg)
 
+    def set_cookies(self, **kwargs):
+        self.session.cookies.update(kwargs)
+
+    def unset_cookies(self, keys):
+        for key in keys:
+            if key in self.session.cookies:
+                self.session.cookies.pop(key)
+
     def _save(self, filename, *args, **kwargs):
         ' requests GET of a "file stream" using current session '
         return self._run(self.session.get, stream=True, filename=filename,
                          *args, **kwargs)
 
     def whoami(self, auth=False):
-        ' quick way of checking the username the instance is working as '
+        '''Request user profile status of currently authenticated user
+        :param auth: (bool) login/authenticate before querying?
+        '''
         if auth:
             self.user_login()
-        else:
-            super(HTTPClient, self).whoami()
+        return super(HTTPClient, self).whoami()
 
 
 # import alias
