@@ -3,17 +3,23 @@
 # Author: "Chris Ward" <cward@redhat.com>
 
 '''
+metrique.cube_api
+~~~~~~~~~~~~~~~~~
+
 This module contains all Cube related api functionality.
 '''
 
 from metriqueu.utils import batch_gen
 
+import logging
+logger = logging.getLogger(__name__)
+
 
 def list_all(self, startswith=None):
     '''
     List all cubes available to the calling client.
 
-    :param string startswith: simple "startswith" filter string
+    :param startswith: string to use in a simple "startswith" query filter
     :returns list: sorted list of cube names
     '''
     return sorted(self._get(startswith))
@@ -30,9 +36,10 @@ def sample_fields(self, cube=None, sample_size=None, query=None, owner=None):
     might not result in a complete list of object fields, since
     some object variants might not be included in the sample queried.
 
-    :param int sample_size: number of random documents to query
-    :param list exclude_fields:
-        List (or csv) of fields to exclude from the results
+    :param cube: cube name
+    :param sample_size: number of random documents to query
+    :param query: `pql` query to filter sample query with
+    :param owner: username of cube owner
     :returns list: sorted list of fields
     '''
     cmd = self.get_cmd(owner, cube)
@@ -41,10 +48,13 @@ def sample_fields(self, cube=None, sample_size=None, query=None, owner=None):
     return sorted(result)
 
 
-def stats(self, cube, owner=None, keys=None):
-    '''
-    Get server reported statistics and other cube details. Optionally,
+def stats(self, cube=None, owner=None, keys=None):
+    '''Get server reported statistics and other cube details. Optionally,
     return only the keys specified, not all the stats.
+
+    :param cube: cube name
+    :param owner: username of cube owner
+    :param keys: return only the specified keys, not all
     '''
     owner = owner or self.config.username
     cmd = self.get_cmd(owner, cube, 'stats')
@@ -61,11 +71,11 @@ def stats(self, cube, owner=None, keys=None):
 
 def drop(self, quiet=False, cube=None, owner=None):
     '''
-    Drop (delete) remote cube.
+    Drop (delete) cube.
 
-    :param string cube: cube name
-    :param bool force: really, do it!
-    :param string owner: username of cube owner
+    :param quiet: ignore exceptions
+    :param owner: username of cube owner
+    :param cube: cube name
     '''
     cmd = self.get_cmd(owner, cube, 'drop')
     try:
@@ -79,10 +89,11 @@ def drop(self, quiet=False, cube=None, owner=None):
 
 def register(self, cube=None, owner=None, quiet=False):
     '''
-    Register a new remote cube
+    Register a new cube.
 
-    :param string cube: cube name
-    :param string owner: username of cube owner
+    :param cube: cube name
+    :param owner: username of cube owner
+    :param quiet: ignore exceptions
     '''
     cmd = self.get_cmd(owner, cube, 'register')
     try:
@@ -100,9 +111,11 @@ def update_role(self, username, cube=None, action='addToSet',
     '''
     Manipulate cube access controls
 
-    :param string action: action to take (addToSet, pull)
-    :param string role:
-        Permission: read, write, admin)
+    :param username: username to manipulate
+    :param cube: cube name
+    :param action: action to take (addToSet, pull)
+    :param role: Permission: read, write, admin)
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'update_role')
     return self._post(cmd, username=username, action=action, role=role)
@@ -111,10 +124,10 @@ def update_role(self, username, cube=None, action='addToSet',
 ######### INDEX #########
 def list_index(self, cube=None, owner=None):
     '''
-    List all remote cube indexes
+    List all cube indexes
 
-    :param string cube: cube name
-    :param string owner: username of cube owner
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'index')
     result = self._get(cmd)
@@ -124,21 +137,17 @@ def list_index(self, cube=None, owner=None):
 def ensure_index(self, key_or_list, name=None, background=None,
                  cube=None, owner=None):
     '''
-    Build a new index on a remote cube.
+    Build a new index on a cube.
 
     Examples:
         + ensure_index('field_name')
         + ensure_index([('field_name', 1), ('other_field_name', -1)])
 
-    :param string/list key_or_list:
-        Either a single key or a list of (key, direction) pairs.
-    :param string name:
-        Custom name to use for this index.
-        If none is given, a name will be generated.
-    :param bool background:
-        If this index should be created in the background.
-    :param string cube: cube name
-    :param string owner: username of cube owner
+    :param key_or_list: A single field or a list of (key, direction) pairs
+    :param name: (optional) Custom name to use for this index
+    :param background: MongoDB should create in the background
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'index')
     return self._post(cmd, ensure=key_or_list, name=name,
@@ -149,73 +158,67 @@ def drop_index(self, index_or_name, cube=None, owner=None):
     '''
     Drops the specified index on this cube.
 
-    :param string/list index_or_name:
-        index (or name of index) to drop
-    :param string cube: cube name
-    :param string owner: username of cube owner
+    :param index_or_name: index (or name of index) to drop
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'index')
     return self._delete(cmd, drop=index_or_name)
 
 
 ######## SAVE/REMOVE ########
-def _save_default(self, objects, start_time, owner, cube):
+def _save_default(self, objects, start_time, owner, cube, autosnap):
     batch_size = self.config.batch_size
     cmd = self.get_cmd(owner, cube, 'save')
     olen = len(objects) if objects else None
     if (batch_size <= 0) or (olen <= batch_size):
-        saved = self._post(cmd, objects=objects, start_time=start_time)
+        saved = self._post(cmd, objects=objects, start_time=start_time,
+                           autosnap=autosnap)
     else:
         saved = []
         k = 0
         for batch in batch_gen(objects, batch_size):
-            _saved = self._post(cmd, objects=batch, start_time=start_time)
+            _saved = self._post(cmd, objects=batch, start_time=start_time,
+                                autosnap=autosnap)
             saved.extend(_saved)
             k += len(batch)
     return saved
 
 
-# FIXME: get rid of start_time? we add this by default during normalization
-# we don't want server to work unnecessarily hard on this... force
-# it to happen client side or metriqued will use the datetime of when
-# the request object was
 def save(self, objects=None, cube=None, owner=None, start_time=None,
-         flush=True):
+         flush=True, autosnap=True):
     '''
     Save a list of objects the given metrique.cube.
     Returns back a list of object ids (_id|_oid) saved.
 
-    :param list objects: list of dictionary-like objects to be stored
-    :param string cube: cube name
-    :param string owner: username of cube owner
-    :param string start_time: ISO format datetime to apply as _start
-                              per object, serverside
-    :param bool flush: flush objects from memory after save
-    :rtype: list - list of object ids saved
+    :param objects: list of dictionary-like objects to be stored
+    :param cube: cube name
+    :param owner: username of cube owner
+    :param start_time: ISO format datetime to apply as _start
+                       per object, serverside
+    :param flush: flush objects from memory after save
+    :param autosnap: rotate _end:None's before saving new objects
+    :returns result: _ids saved
     '''
-    if objects is None:
-        objects = self.objects
     if not objects:
-        self.logger.info("... No objects to save")
-        self.result = []
+        logger.info("... No objects to save")
+        result = []
     else:
-        self.logger.info("Saving %s objects" % len(objects))
-        # support only list of dicts
-        saved = _save_default(self, objects, start_time, owner, cube)
-        self.logger.info("... Saved %s NEW docs" % len(saved))
-        self.result = saved
+        logger.info("Saving %s objects" % len(objects))
+        result = _save_default(self, objects, start_time, owner, cube,
+                               autosnap)
         if flush:
             self.flush()
-    return
+    return result
 
 
 def rename(self, new_name, cube=None, owner=None):
     '''
-    Rename a remote cube.
+    Rename a cube.
 
-    :param string new_name: new cube name
-    :param string cube: cube name
-    :param string owner: username of cube owner
+    :param new_name: new cube name
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'rename')
     result = self._post(cmd, new_name=new_name)
@@ -226,12 +229,11 @@ def rename(self, new_name, cube=None, owner=None):
 
 def remove(self, query, date=None, cube=None, owner=None):
     '''
-    Remove objects from a remote cube.
+    Remove objects from a cube.
 
-    :param list ids: list of object ids to remove
-    :param bool backup: return the documents removed to client?
-    :param string cube: cube name
-    :param string owner: username of cube owner
+    :param query: `pql` query to filter sample query with
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'remove')
     result = self._delete(cmd, query=query, date=date)
@@ -240,10 +242,11 @@ def remove(self, query, date=None, cube=None, owner=None):
 
 def export(self, filename, cube=None, owner=None):
     '''
-    Export a remote cube to compressed (gzip) json
+    Export a cube to compressed (gzip) json
 
-    :param string cube: cube name
-    :param string owner: username of cube owner
+    :param filename: path/filename of results export.tgz file
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'export')
     return self._save(cmd=cmd, filename=filename)
