@@ -3,20 +3,26 @@
 # Author: "Chris Ward" <cward@redhat.com>
 
 '''
+metrique.query_api
+~~~~~~~~~~~~~~~~~~
+
 This module contains all the query and aggregation
 related api functionality.
 '''
 
 from metrique.result import Result
 
+import logging
+logger = logging.getLogger(__name__)
+
 
 def aggregate(self, pipeline, cube=None, owner=None):
     '''
     Run a pql mongodb aggregate pipeline on remote cube
 
-    :param list pipeline: The aggregation pipeline. $match, $project, etc.
-    :param string cube: name of cube to work with
-    :param string owner: owner of cube
+    :param pipeline: The aggregation pipeline. $match, $project, etc.
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'aggregate')
     result = self._get(cmd, pipeline=pipeline)
@@ -28,10 +34,12 @@ def count(self, query=None, date=None, cube=None, owner=None):
     Run a pql mongodb based query on the given cube and return only
     the count of resulting matches.
 
-    :param string query: The query in pql
-    :param string date: Date (date range) that should be queried
-    :param string cube: name of cube to work with
-    :param string owner: owner of cube
+    :param query: The query in pql
+    :param date: date (metrique date range) that should be queried
+                 If date==None then the most recent versions of the
+                 objects will be queried.
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'count')
     return self._get(cmd, query=query, date=date)
@@ -43,20 +51,20 @@ def find(self, query=None, fields=None, date=None, sort=None, one=False,
     '''
     Run a pql mongodb based query on the given cube.
 
-    :param string query: The query in pql
-    :param string fields: Fields that should be returned (comma-separated)
-    :param string date: Date (date range) that should be queried.
-                        If date==None then the most recent versions of the
-                        objects will be queried.
-    :param bool explain: return execution plan instead of results
-    :param boolean merge_versions: merge versions where fields values equal
-    :param bool one: return back only first matching object
-    :param bool sort: return back results sorted
-    :param bool raw: return back raw JSON results rather than pandas dataframe
-    :param int skip: number of results matched to skip and not return
-    :param int limit: number of results matched to return of total found
-    :param string cube: name of cube to work with
-    :param string owner: owner of cube
+    :param query: The query in pql
+    :param fields: Fields that should be returned (comma-separated)
+    :param date: date (metrique date range) that should be queried.
+                 If date==None then the most recent versions of the
+                 objects will be queried.
+    :param explain: return execution plan instead of results
+    :param merge_versions: merge versions where fields values equal
+    :param one: return back only first matching object
+    :param sort: return back results sorted
+    :param raw: return back raw JSON results rather than pandas dataframe
+    :param skip: number of results matched to skip and not return
+    :param limit: number of results matched to return of total found
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'find')
     result = self._get(cmd, query=query, fields=fields,
@@ -71,11 +79,11 @@ def history(self, query, by_field=None, date_list=None, cube=None, owner=None):
     Run a pql mongodb based query on the given cube and return back the
     aggregate historical counts of matching results.
 
-    :param string query: The query in pql
-    :param string by_field: Which field to slice/dice and aggregate from
-    :param string date: list of dates that should be used to bin the results
-    :param string cube: name of cube to work with
-    :param string owner: owner of cube
+    :param query: The query in pql
+    :param by_field: Which field to slice/dice and aggregate from
+    :param date: list of dates that should be used to bin the results
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'history')
     return self._get(cmd, query=query, by_field=by_field, date_list=date_list)
@@ -86,15 +94,14 @@ def deptree(self, field, oids, date=None, level=None, cube=None, owner=None):
     Dependency tree builder. Recursively fetchs objects that
     are children of the initial set of parent object ids provided.
 
-    :param string field: Field that contains the 'parent of' data
-    :param list oids: Object oids to build depedency tree for
-    :param string date:
-        Date (date range) that should be queried.
-        If date==None then the most recent versions of the objects
-        will be queried.
-    :param integer level: limit depth of recursion
-    :param string cube: name of cube to work with
-    :param string owner: owner of cube
+    :param field: Field that contains the 'parent of' data
+    :param oids: Object oids to build depedency tree for
+    :param date: date (metrique date range) that should be queried.
+                 If date==None then the most recent versions of the
+                 objects will be queried.
+    :param level: limit depth of recursion
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'deptree')
     result = self._get(cmd, field=field,
@@ -102,18 +109,17 @@ def deptree(self, field, oids, date=None, level=None, cube=None, owner=None):
     return sorted(result)
 
 
-def distinct(self, field, cube=None, owner=None):
+def distinct(self, field, query=None, date=None, cube=None, owner=None):
     '''
     Return back a distinct (unique) list of field values
     across the entire cube dataset
 
-    :param string field:
-        Field to get distinct token values from
-    :param string cube: name of cube to work with
-    :param string owner: owner of cube
+    :param field: field to get distinct token values from
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'distinct')
-    result = self._get(cmd, field=field)
+    result = self._get(cmd, field=field, query=query, date=date)
     return sorted(result)
 
 
@@ -122,15 +128,15 @@ def sample(self, sample_size=1, fields=None, date=None, raw=False,
     '''
     Draws a sample of objects at random from the cube.
 
-    :param integer sample_size: Size of the sample
+    :param sample_size: Size of the sample
     :param fields: Fields that should be returned
-    :param string date: Date (date range) that should be queried
-                        If date==None then the most recent versions of
-                        the objects will be queried
-    :param boolean raw: if True, then return result as a dictionary
-    :param string query: query used to filter sampleset
-    :param string cube: name of cube to work with
-    :param string owner: owner of cube
+    :param date: date (metrique date range) that should be queried
+                       If date==None then the most recent versions of
+                       the objects will be queried
+    :param raw: if True, then return result as a dictionary
+    :param query: query used to filter sampleset
+    :param cube: cube name
+    :param owner: username of cube owner
     '''
     cmd = self.get_cmd(owner, cube, 'sample')
     result = self._get(cmd, sample_size=sample_size,
